searchData={"items":[{"type":"module","title":"Chord","doc":"Chord: A powerful library for managing real-time contexts with efficiency and flexibility.\n\nThe Chord module serves as the entry point for the library, providing a high-level API\nto manage contexts, deltas, cleanup, and integrations with various storage backends.\nIt simplifies state synchronization and lifecycle management for real-time applications.","ref":"Chord.html"},{"type":"module","title":"Key Features - Chord","doc":"- **Context Management**: Create, update, synchronize, and delete application contexts.\n- **Delta Tracking**: Efficiently track changes to minimize data transfer.\n- **Flexible Backends**: Support for in-memory (ETS) and distributed (Redis) storage.\n- **Context Export**: Easily export contexts to external storage.\n- **Customizable Cleanup**: Automated or manual cleanup of expired data.\n- **Partial Updates**: Apply updates to specific fields within a context.\n- **External Context Provider**: Restore contexts from external sources when needed.","ref":"Chord.html#module-key-features"},{"type":"module","title":"Configuration - Chord","doc":"Chord is designed to be configurable via the application environment. Below are the\nconfiguration options available:\n\n    config :chord,\n      backend: Chord.Backend.ETS, # Backend for storing contexts and deltas (ETS or Redis)\n      context_auto_delete: true, # Automatically delete expired contexts\n      context_ttl: :timer.hours(6), # Time-to-live for contexts\n      delta_ttl: :timer.hours(3), # Time-to-live for deltas\n      delta_threshold: 100, # Maximum number of deltas to retain per context\n      delta_formatter: Chord.Delta.Formatter.Default, # Formatter for deltas\n      time_provider: MyApp.TimeProvider, # Custom time provider for timestamping\n      export_callback: &MyApp.ContextExporter.export/1, # Callback for exporting contexts\n      context_external_provider: &MyApp.ExternalState.fetch_context/1 # Restore context from external storage","ref":"Chord.html#module-configuration"},{"type":"module","title":"Common Options - Chord","doc":"The following options are shared across several functions in this module and related modules:\n\n  - :context_id - The identifier for the context.\n  - :version - The version number for tracking state changes.\n  - :inserted_at - The timestamp when the context or delta was created.\n  - :limit - The maximum number of contexts or deltas to fetch.\n  - :offset - The number of entries to skip when fetching.\n  - :order - The sort order for results (:asc or :desc).","ref":"Chord.html#module-common-options"},{"type":"module","title":"Example Usage - Chord","doc":"Here's how you can use Chord in your application:\n\n    # Set a context\n    {:ok, %{context: updated_context, delta: delta}} =\n      Chord.set_context(\"call:123\", %{status: \"active\"})\n\n    # Get the current context\n    {:ok, %{context_id: context_id, context: context, version: version, inserted_at: inserted_at}} = Chord.get_context(\"call:123\")\n\n    # Apply a partial update to a context\n    {:ok, %{context: updated_context, delta: delta}} =\n      Chord.update_context(\"call:123\", %{status: \"ended\"})\n\n    # Synchronize context with a client\n    case Chord.sync_context(\"call:123\", client_version) do\n      {:full_context, context} -> :send_full_context\n      {:delta, delta} -> :send_delta\n      {:no_change, version} -> :no_change\n    end\n\n    # Restore a context from external storage\n    {:ok, context_data} = Chord.restore_context(\"call:123\")\n\n    # Export a context to external storage\n    :ok = Chord.export_context(\"call:123\")\n\n    # Trigger cleanup\n    Chord.cleanup(limit: 10)\n\n    # Start the Cleanup.Server for periodic cleanup\n    {:ok, _pid} = Chord.start_cleanup_server(interval: :timer.minutes(30))","ref":"Chord.html#module-example-usage"},{"type":"function","title":"Chord.cleanup/1","doc":"Triggers a one-time cleanup for stale contexts and deltas.","ref":"Chord.html#cleanup/1"},{"type":"function","title":"Parameters - Chord.cleanup/1","doc":"- `opts` (Keyword.t): Optional parameters for filtering contexts and deltas (see \"Common Options\").","ref":"Chord.html#cleanup/1-parameters"},{"type":"function","title":"Returns - Chord.cleanup/1","doc":"- `:ok` on success.","ref":"Chord.html#cleanup/1-returns"},{"type":"function","title":"Chord.delete_context/1","doc":"Deletes the global context, including all deltas.","ref":"Chord.html#delete_context/1"},{"type":"function","title":"Parameters - Chord.delete_context/1","doc":"- `context_id` - The identifier for the context.","ref":"Chord.html#delete_context/1-parameters"},{"type":"function","title":"Returns - Chord.delete_context/1","doc":"- `:ok` on success.","ref":"Chord.html#delete_context/1-returns"},{"type":"function","title":"Chord.export_context/1","doc":"Exports the active context for a given identifier to external storage using the configured export callback.","ref":"Chord.html#export_context/1"},{"type":"function","title":"Parameters - Chord.export_context/1","doc":"- `context_id` - The identifier for the context.","ref":"Chord.html#export_context/1-parameters"},{"type":"function","title":"Returns - Chord.export_context/1","doc":"- `:ok` if the context is successfully exported.\n- `{:error, :not_found}` if the context does not exist.","ref":"Chord.html#export_context/1-returns"},{"type":"function","title":"Chord.get_context/1","doc":"Retrieves the current global context for a given identifier.","ref":"Chord.html#get_context/1"},{"type":"function","title":"Parameters - Chord.get_context/1","doc":"- `context_id` - The identifier for the context.","ref":"Chord.html#get_context/1-parameters"},{"type":"function","title":"Returns - Chord.get_context/1","doc":"- `{:ok, map()}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.html#get_context/1-returns"},{"type":"function","title":"Chord.restore_context/1","doc":"Restores a context from an external provider to the current backend.","ref":"Chord.html#restore_context/1"},{"type":"function","title":"Parameters - Chord.restore_context/1","doc":"- `context_id` (any): The ID of the context to restore.","ref":"Chord.html#restore_context/1-parameters"},{"type":"function","title":"Returns - Chord.restore_context/1","doc":"- `{:ok, map()}` on success.\n  - `{:error, :not_found}` if the context is missing in external storage.","ref":"Chord.html#restore_context/1-returns"},{"type":"function","title":"Chord.set_context/2","doc":"Sets the global context and records deltas.","ref":"Chord.html#set_context/2"},{"type":"function","title":"Parameters - Chord.set_context/2","doc":"- `context_id` - The identifier for the context.\n  - `new_context` - The new context to be stored.","ref":"Chord.html#set_context/2-parameters"},{"type":"function","title":"Returns - Chord.set_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.html#set_context/2-returns"},{"type":"function","title":"Chord.start_cleanup_server/1","doc":"Starts the Cleanup.Server for periodic cleanup.","ref":"Chord.html#start_cleanup_server/1"},{"type":"function","title":"Parameters - Chord.start_cleanup_server/1","doc":"- `opts` - Options for the Cleanup.Server. Examples include:\n    - `:interval` - The interval (in milliseconds) between cleanup executions.\n    - `:backend_opts` - A keyword list of options passed to backend listing functions (see \"Common Options\").","ref":"Chord.html#start_cleanup_server/1-parameters"},{"type":"function","title":"Returns - Chord.start_cleanup_server/1","doc":"- `{:ok, pid}` on success.","ref":"Chord.html#start_cleanup_server/1-returns"},{"type":"function","title":"Chord.stop_cleanup_server/0","doc":"Stops the CleanupServer if running.","ref":"Chord.html#stop_cleanup_server/0"},{"type":"function","title":"Returns - Chord.stop_cleanup_server/0","doc":"- `:ok` on success.","ref":"Chord.html#stop_cleanup_server/0-returns"},{"type":"function","title":"Chord.sync_context/2","doc":"Synchronizes the context for a client based on its current version.","ref":"Chord.html#sync_context/2"},{"type":"function","title":"Parameters - Chord.sync_context/2","doc":"- `context_id` - The identifier for the context.\n  - `client_version` - The last known version for the client.","ref":"Chord.html#sync_context/2-parameters"},{"type":"function","title":"Returns - Chord.sync_context/2","doc":"- `{:full_context, map()}` if a full context is sent.\n- `{:delta, map()}` if deltas are sent.\n- `{:no_change, version}` if no changes are needed.","ref":"Chord.html#sync_context/2-returns"},{"type":"function","title":"Chord.update_cleanup_backend_opts/1","doc":"Updates the backend options for the cleanup server at runtime.","ref":"Chord.html#update_cleanup_backend_opts/1"},{"type":"function","title":"Parameters - Chord.update_cleanup_backend_opts/1","doc":"- `new_opts` (Keyword.t): The new backend options.","ref":"Chord.html#update_cleanup_backend_opts/1-parameters"},{"type":"function","title":"Chord.update_cleanup_interval/1","doc":"Updates the cleanup server's interval at runtime.","ref":"Chord.html#update_cleanup_interval/1"},{"type":"function","title":"Parameters - Chord.update_cleanup_interval/1","doc":"- `new_interval` (integer): The new interval in milliseconds.","ref":"Chord.html#update_cleanup_interval/1-parameters"},{"type":"function","title":"Chord.update_context/2","doc":"Partially updates the global context for a given identifier.","ref":"Chord.html#update_context/2"},{"type":"function","title":"Parameters - Chord.update_context/2","doc":"- `context_id` - The identifier for the context.\n  - `changes` - A map of fields to be updated in the existing context.","ref":"Chord.html#update_context/2-parameters"},{"type":"function","title":"Returns - Chord.update_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.html#update_context/2-returns"},{"type":"behaviour","title":"Chord.Backend.Behaviour","doc":"Defines the behavior for backends used by Chord.\n\nThis module specifies the contract that any backend implementation must adhere to.\nIt provides a consistent interface for managing context and delta data, supporting\noperations for context storage, delta tracking, and cleanup.","ref":"Chord.Backend.Behaviour.html"},{"type":"behaviour","title":"Key Concepts - Chord.Backend.Behaviour","doc":"- **Context Operations:** Manage context data associated with a unique `context_id`.\n- **Delta Operations:** Track and retrieve changes (deltas) associated with contexts.\n- **Listing Operations:** Enable filtering and listing of contexts and deltas.\n- **Cleanup:** Support for deleting old or unnecessary data to maintain performance.\n\nDevelopers can implement this behavior to provide custom backends (e.g., Redis, Database).","ref":"Chord.Backend.Behaviour.html#module-key-concepts"},{"type":"behaviour","title":"Example Usage - Chord.Backend.Behaviour","doc":"To implement a custom backend, define a module and implement the callbacks specified here:\n\n    defmodule MyApp.CustomBackend do\n      @behaviour Chord.Backend.Behaviour\n\n      def set_context(context_id, context, version), do: # implementation\n      def get_context(context_id), do: # implementation\n      def delete_context(context_id), do: # implementation\n      # Implement all other callbacks...\n    end\n\nOnce implemented, configure the backend in your application:\n\n    config :chord, :backend, MyApp.CustomBackend","ref":"Chord.Backend.Behaviour.html#module-example-usage"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_context/1","doc":"Deletes the context for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_context/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_context/1","doc":"- `context_id` (any): The identifier for the context to delete.","ref":"Chord.Backend.Behaviour.html#c:delete_context/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_context/1","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_context/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_deltas_by_time/2","doc":"Deletes deltas older than a specified time for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_by_time/2"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_deltas_by_time/2","doc":"- `context_id` (any): The identifier for the context.\n  - `older_than_time` (integer): The timestamp threshold.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_by_time/2-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_deltas_by_time/2","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_by_time/2-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_deltas_exceeding_threshold/2","doc":"Deletes deltas exceeding a specified version threshold for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_exceeding_threshold/2"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_deltas_exceeding_threshold/2","doc":"- `context_id` (any): The identifier for the context.\n  - `version_threshold` (integer): The maximum number of versions to retain.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_exceeding_threshold/2-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_deltas_exceeding_threshold/2","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_exceeding_threshold/2-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.delete_deltas_for_context/1","doc":"Deletes all deltas for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_for_context/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.delete_deltas_for_context/1","doc":"- `context_id` (any): The identifier for the context.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_for_context/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.delete_deltas_for_context/1","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:delete_deltas_for_context/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.get_context/1","doc":"Retrieves the context and version for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:get_context/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.get_context/1","doc":"- `context_id` (any): The identifier for the context.","ref":"Chord.Backend.Behaviour.html#c:get_context/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.get_context/1","doc":"- `{:ok, %{context_id: any(), context: map(), version: integer(), inserted_at: integer()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:get_context/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.get_deltas/2","doc":"Retrieves deltas for a given `context_id` greater than a specified `client_version`.","ref":"Chord.Backend.Behaviour.html#c:get_deltas/2"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.get_deltas/2","doc":"- `context_id` (any): The identifier for the context.\n  - `client_version` (integer): The version from which to retrieve deltas.","ref":"Chord.Backend.Behaviour.html#c:get_deltas/2-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.get_deltas/2","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` if no deltas are found or an error occurs.","ref":"Chord.Backend.Behaviour.html#c:get_deltas/2-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.list_contexts/1","doc":"Lists contexts with optional filters and pagination.","ref":"Chord.Backend.Behaviour.html#c:list_contexts/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.list_contexts/1","doc":"- `opts` (Keyword.t): Options for filtering and pagination. Common options include:\n    - `:context_id` - Filter by context identifier.\n    - `:version` - Filter by version.\n    - `:inserted_at` - Filter by insertion timestamp.\n    - `:limit` - Maximum number of results to return.\n    - `:offset` - Number of results to skip.","ref":"Chord.Backend.Behaviour.html#c:list_contexts/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.list_contexts/1","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:list_contexts/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.list_contexts_with_delta_counts/1","doc":"Lists contexts along with their delta counts.","ref":"Chord.Backend.Behaviour.html#c:list_contexts_with_delta_counts/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.list_contexts_with_delta_counts/1","doc":"- `opts` (Keyword.t): Options for filtering and pagination.","ref":"Chord.Backend.Behaviour.html#c:list_contexts_with_delta_counts/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.list_contexts_with_delta_counts/1","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:list_contexts_with_delta_counts/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.list_deltas/1","doc":"Lists deltas with optional filters and pagination.","ref":"Chord.Backend.Behaviour.html#c:list_deltas/1"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.list_deltas/1","doc":"- `opts` (Keyword.t): Options for filtering and pagination. Common options include:\n    - `:context_id` - Filter by context identifier.\n    - `:version` - Filter by version.\n    - `:inserted_at` - Filter by insertion timestamp.\n    - `:limit` - Maximum number of results to return.\n    - `:offset` - Number of results to skip.","ref":"Chord.Backend.Behaviour.html#c:list_deltas/1-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.list_deltas/1","doc":"- `{:ok, list(map())}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:list_deltas/1-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.set_context/3","doc":"Sets the context for a given `context_id`.","ref":"Chord.Backend.Behaviour.html#c:set_context/3"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.set_context/3","doc":"- `context_id` (any): The identifier for the context.\n  - `context` (map): The context data to store.\n  - `version` (integer): The version number associated with the context.","ref":"Chord.Backend.Behaviour.html#c:set_context/3-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.set_context/3","doc":"- `{:ok, %{context_id: any(), context: map(), version: integer(), inserted_at: integer()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:set_context/3-returns"},{"type":"callback","title":"Chord.Backend.Behaviour.set_delta/3","doc":"Sets a delta for a given `context_id` and `version`.","ref":"Chord.Backend.Behaviour.html#c:set_delta/3"},{"type":"callback","title":"Parameters - Chord.Backend.Behaviour.set_delta/3","doc":"- `context_id` (any): The identifier for the context.\n  - `delta` (map): The delta data to store.\n  - `version` (integer): The version number associated with the delta.","ref":"Chord.Backend.Behaviour.html#c:set_delta/3-parameters"},{"type":"callback","title":"Returns - Chord.Backend.Behaviour.set_delta/3","doc":"- `{:ok, %{context_id: any(), delta: map(), version: integer(), inserted_at: integer()}}` on success.\n- `{:error, term()}` on failure.","ref":"Chord.Backend.Behaviour.html#c:set_delta/3-returns"},{"type":"module","title":"Chord.Backend.ETS","doc":"ETS-based backend for Chord, providing context-level state and delta management\nwith versioning support. Both tables use `:ordered_set` to leverage efficient\nkey ordering for listing and cleanup operations.","ref":"Chord.Backend.ETS.html"},{"type":"module","title":"Tables - Chord.Backend.ETS","doc":"- `@context_table`: Stores context state with a composite key `{context_id, inserted_at}`.\n- `@context_history_table`: Stores deltas with a composite key `{context_id, version}`.","ref":"Chord.Backend.ETS.html#module-tables"},{"type":"module","title":"Configuration - Chord.Backend.ETS","doc":"This backend can be used out-of-the-box with the following settings:\n    config :chord, :backend, Chord.Backend.ETS","ref":"Chord.Backend.ETS.html#module-configuration"},{"type":"module","title":"Chord.Backend.Redis","doc":"Redis-based backend for Chord, providing context-level state and delta management\nwith versioning support.","ref":"Chord.Backend.Redis.html"},{"type":"module","title":"Configuration - Chord.Backend.Redis","doc":"To use Redis as a backend, follow these steps:\n\n1. Start a Redis connection process using `Redix` with your desired configuration:\n\n  ```elixir\n  {:ok, _} = Redix.start_link(\"redis://localhost:6379\", name: :my_redis)\n  ```\n\n2. Set the Redis client in the application environment:\n\n  ```elixir\n  Application.put_env(:chord, :redis_client, :my_redis)\n  ```","ref":"Chord.Backend.Redis.html#module-configuration"},{"type":"module","title":"Chord.Cleanup","doc":"This module provides functionality to clean up data that is no longer needed,\nsuch as old contexts or deltas, ensuring efficient use of storage and memory.","ref":"Chord.Cleanup.html"},{"type":"module","title":"Key Features - Chord.Cleanup","doc":"- **Time-Based Cleanup**: Periodically remove stale contexts or deltas based on configurable time-to-live (TTL) values.\n- **Threshold-Based Cleanup**: Retain only the latest N deltas per context, defined by the `:delta_threshold` configuration.\n- **Periodic Cleanup**: A convenient API for batch cleanup of multiple contexts and deltas.","ref":"Chord.Cleanup.html#module-key-features"},{"type":"module","title":"Configuration - Chord.Cleanup","doc":"- `:context_auto_delete` (default: false): Whether to enable automatic context deletion during cleanup.\n- `:context_ttl` (default: nil): Time-to-live for context entries.\n- `:delta_ttl` (default: 24 hours): Time-to-live for deltas.\n- `:delta_threshold` (default: 100): Maximum number of deltas to retain per context.\n- `:backend`: The backend module used for context and delta storage (default: `Chord.Backend.ETS`).","ref":"Chord.Cleanup.html#module-configuration"},{"type":"module","title":"Example Usage - Chord.Cleanup","doc":"iex> Chord.Cleanup.periodic_cleanup(limit: 10)\n    :ok","ref":"Chord.Cleanup.html#module-example-usage"},{"type":"function","title":"Chord.Cleanup.periodic_cleanup/1","doc":"Performs periodic cleanup for contexts and deltas.\n\nThis function fetches all contexts and deltas from the backend, checks their times against\nthe configured TTL values, and removes stale entries.","ref":"Chord.Cleanup.html#periodic_cleanup/1"},{"type":"function","title":"Parameters - Chord.Cleanup.periodic_cleanup/1","doc":"- `opts` (Keyword.t): Optional parameters for fetching contexts and deltas (see \"Common Options\" in the `Chord` module).","ref":"Chord.Cleanup.html#periodic_cleanup/1-parameters"},{"type":"function","title":"Returns - Chord.Cleanup.periodic_cleanup/1","doc":"- `:ok` after performing the cleanup.","ref":"Chord.Cleanup.html#periodic_cleanup/1-returns"},{"type":"function","title":"Example - Chord.Cleanup.periodic_cleanup/1","doc":"iex> Chord.Cleanup.periodic_cleanup(limit: 10)\n    :ok","ref":"Chord.Cleanup.html#periodic_cleanup/1-example"},{"type":"module","title":"Chord.Cleanup.Server","doc":"Periodic cleanup server for stale contexts and deltas.\n\nThis server periodically triggers the cleanup logic defined in `Chord.Cleanup`,\nremoving stale entries from the backend.","ref":"Chord.Cleanup.Server.html"},{"type":"module","title":"State - Chord.Cleanup.Server","doc":"The server's state includes:\n  - `:interval` - The interval (in milliseconds) between cleanup executions.\n  - `:backend_opts` - A keyword list of options passed to backend listing functions.","ref":"Chord.Cleanup.Server.html#module-state"},{"type":"module","title":"Example - Chord.Cleanup.Server","doc":"Chord.Cleanup.Server.start_link(interval: :timer.minutes(30), backend_opts: [limit: 100])","ref":"Chord.Cleanup.Server.html#module-example"},{"type":"function","title":"Chord.Cleanup.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Chord.Cleanup.Server.html#child_spec/1"},{"type":"function","title":"Chord.Cleanup.Server.start_link/1","doc":"Starts the cleanup server with the specified options.","ref":"Chord.Cleanup.Server.html#start_link/1"},{"type":"function","title":"Options - Chord.Cleanup.Server.start_link/1","doc":"- `:interval` (integer): Time interval in milliseconds for periodic cleanup (default: 1 hour).\n  - `:backend_opts` (Keyword.t): Options passed to backend listing functions (see \"Common Options\" in the `Chord` module).","ref":"Chord.Cleanup.Server.html#start_link/1-options"},{"type":"function","title":"Example - Chord.Cleanup.Server.start_link/1","doc":"iex> Chord.Cleanup.Server.start_link(interval: :timer.minutes(30), backend_opts: [limit: 50])\n    {:ok, pid}","ref":"Chord.Cleanup.Server.html#start_link/1-example"},{"type":"function","title":"Chord.Cleanup.Server.update_backend_opts/2","doc":"Updates the backend options for periodic cleanup at runtime.","ref":"Chord.Cleanup.Server.html#update_backend_opts/2"},{"type":"function","title":"Parameters - Chord.Cleanup.Server.update_backend_opts/2","doc":"- `new_opts` (Keyword.t): The new backend options.\n  - `name` (atom): The name of the `GenServer` instance. Defaults to `__MODULE__`.","ref":"Chord.Cleanup.Server.html#update_backend_opts/2-parameters"},{"type":"function","title":"Returns - Chord.Cleanup.Server.update_backend_opts/2","doc":"- `:ok` if the backend options were successfully updated.","ref":"Chord.Cleanup.Server.html#update_backend_opts/2-returns"},{"type":"function","title":"Chord.Cleanup.Server.update_interval/2","doc":"Updates the interval for periodic cleanup at runtime.","ref":"Chord.Cleanup.Server.html#update_interval/2"},{"type":"function","title":"Parameters - Chord.Cleanup.Server.update_interval/2","doc":"- `new_interval` (integer): The new interval in milliseconds.\n  - `name` (atom): The name of the `GenServer` instance. Defaults to `__MODULE__`.","ref":"Chord.Cleanup.Server.html#update_interval/2-parameters"},{"type":"function","title":"Returns - Chord.Cleanup.Server.update_interval/2","doc":"- `:ok` if the interval was successfully updated.","ref":"Chord.Cleanup.Server.html#update_interval/2-returns"},{"type":"module","title":"Chord.Context.Manager","doc":"Orchestrates context synchronization, delta calculation, and backend interactions.\n\nThe `Chord.Context.Manager` module is responsible for the core operations in Chord,\nincluding context management, delta tracking, and backend interactions. It provides\na flexible and efficient system for managing state and deltas, designed to work seamlessly\nwith in-memory or external backends, offering users customizable workflows for their specific needs.","ref":"Chord.Context.Manager.html"},{"type":"module","title":"Features - Chord.Context.Manager","doc":"- **Context Management:** Handles retrieval, updates, and deletions of contexts efficiently.\n- **Delta Tracking:** Automatically calculates and stores deltas between context updates.\n- **Synchronization:** Supports syncing contexts and deltas to clients based on their known versions.\n- **Exporting Contexts:** Allows exporting active contexts to external storage using a configurable callback.\n- **Flexible Backends:** Compatible with built-in ETS or Redis backends, as well as custom backends.\n- **External Context Integration:** Supports fetching contexts from external storage when needed.","ref":"Chord.Context.Manager.html#module-features"},{"type":"module","title":"Use Cases - Chord.Context.Manager","doc":"This module is designed for applications where efficient state and delta management\nis critical, such as:\n\n- Real-time collaborative applications (e.g., document editing, multiplayer games).\n- Communication platforms (e.g., chat systems, call management).\n- Systems requiring versioned state synchronization.\n\nDevelopers can leverage `Chord.Context.Manager` to manage contexts with minimal boilerplate,\nwhile retaining the flexibility to adapt the behavior through configuration and callbacks.","ref":"Chord.Context.Manager.html#module-use-cases"},{"type":"module","title":"Configuration - Chord.Context.Manager","doc":"The behavior of this module is driven by the following configuration options:\n\n    config :chord,\n      backend: Chord.Backend.ETS, # Backend to use for storing contexts and deltas.\n      delta_threshold: 100, # Threshold for determining when to send full contexts.\n      export_callback: &MyApp.ContextExporter.export/1, # Callback for exporting contexts.\n      context_external_provider: &MyApp.ExternalProvider.fetch_context/1 # Function for fetching external contexts.","ref":"Chord.Context.Manager.html#module-configuration"},{"type":"function","title":"Chord.Context.Manager.delete_context/1","doc":"Deletes the context and its associated deltas.","ref":"Chord.Context.Manager.html#delete_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.delete_context/1","doc":"- `context_id` (any): The ID of the context to delete.","ref":"Chord.Context.Manager.html#delete_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.delete_context/1","doc":"- `:ok` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Context.Manager.html#delete_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.export_context/1","doc":"Exports the context for a given identifier to an external storage.","ref":"Chord.Context.Manager.html#export_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.export_context/1","doc":"- `context_id` (any): The ID of the context to export.","ref":"Chord.Context.Manager.html#export_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.export_context/1","doc":"- `:ok` on success.\n  - `{:error, :not_found}` if the context does not exist.","ref":"Chord.Context.Manager.html#export_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.get_context/1","doc":"Retrieves the current context for a given identifier.","ref":"Chord.Context.Manager.html#get_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.get_context/1","doc":"- `context_id` (any): The ID of the context to be retrieved.","ref":"Chord.Context.Manager.html#get_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.get_context/1","doc":"- `{:ok, map()}` if context exists.\n  - `{:error, term()}` if no context is available.","ref":"Chord.Context.Manager.html#get_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.restore_context/1","doc":"Restores a context from an external provider to the current backend.","ref":"Chord.Context.Manager.html#restore_context/1"},{"type":"function","title":"Parameters - Chord.Context.Manager.restore_context/1","doc":"- `context_id` (any): The ID of the context to restore.","ref":"Chord.Context.Manager.html#restore_context/1-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.restore_context/1","doc":"- `{:ok, map()}` on success.\n  - `{:error, :not_found}` if the context is missing in external storage.","ref":"Chord.Context.Manager.html#restore_context/1-returns"},{"type":"function","title":"Chord.Context.Manager.set_context/2","doc":"Updates the global context for a given identifier and calculates deltas.","ref":"Chord.Context.Manager.html#set_context/2"},{"type":"function","title":"Parameters - Chord.Context.Manager.set_context/2","doc":"- `context_id` (any): The ID of the context to update.\n  - `new_context` (map): The full updated context to set.","ref":"Chord.Context.Manager.html#set_context/2-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.set_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Context.Manager.html#set_context/2-returns"},{"type":"function","title":"Chord.Context.Manager.sync_context/2","doc":"Synchronizes the context for a specific identifier.\n\nDetermines whether the client receives the full context, deltas, or no changes\nbased on the client's known version.","ref":"Chord.Context.Manager.html#sync_context/2"},{"type":"function","title":"Parameters - Chord.Context.Manager.sync_context/2","doc":"- `context_id` (any): The ID of the context to synchronize.\n  - `client_version` (integer | nil): The version of the context known to the client.","ref":"Chord.Context.Manager.html#sync_context/2-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.sync_context/2","doc":"- `{:full_context, map()}` if the client should receive the full context.\n  - `{:delta, map()}` if the client should receive deltas.\n  - `{:no_change, integer()}` if the client has the latest version.","ref":"Chord.Context.Manager.html#sync_context/2-returns"},{"type":"function","title":"Chord.Context.Manager.update_context/2","doc":"Partially updates the context for a given identifier and calculates deltas.\n\nOnly the provided fields in `changes` are updated in the existing context.","ref":"Chord.Context.Manager.html#update_context/2"},{"type":"function","title":"Parameters - Chord.Context.Manager.update_context/2","doc":"- `context_id` (any): The ID of the context to update.\n  - `changes` (map): A map of fields to be updated.","ref":"Chord.Context.Manager.html#update_context/2-parameters"},{"type":"function","title":"Returns - Chord.Context.Manager.update_context/2","doc":"- `{:ok, %{context: map(), delta: map()}}` on success.\n  - `{:error, term()}` on failure.","ref":"Chord.Context.Manager.html#update_context/2-returns"},{"type":"module","title":"Chord.Delta","doc":"Provides utilities for calculating, merging, and formatting context deltas.\n\nThis module contains functions to:\n- Compute the differences (deltas) between two contexts.\n- Merge multiple deltas into a single representation.\n- Format deltas for communication or storage purposes.\n\nThe formatting of deltas is customizable. Developers can define their own formatter module\nimplementing the `Chord.Delta.Formatter.Behaviour` and configure it in their application.\nBy default, the library uses `Chord.Delta.Formatter.Default`.","ref":"Chord.Delta.html"},{"type":"function","title":"Chord.Delta.calculate_delta/2","doc":"Calculates the delta between two contexts.\n\nGiven a `current_context` and a `new_context`, this function determines the differences:\n- Keys present in `new_context` but not in `current_context` are marked as `:added`.\n- Keys present in both but with differing values are marked as `:modified`.\n- Keys present in `current_context` but absent in `new_context` are marked as `:removed`.","ref":"Chord.Delta.html#calculate_delta/2"},{"type":"function","title":"Parameters - Chord.Delta.calculate_delta/2","doc":"- `current_context` (map): The original context.\n  - `new_context` (map): The updated context.","ref":"Chord.Delta.html#calculate_delta/2-parameters"},{"type":"function","title":"Returns - Chord.Delta.calculate_delta/2","doc":"- (map): A map representing the delta, where each key maps to a change descriptor.","ref":"Chord.Delta.html#calculate_delta/2-returns"},{"type":"function","title":"Example - Chord.Delta.calculate_delta/2","doc":"iex> current_context = %{a: 1, b: 2, c: 3}\n    iex> new_context = %{a: 1, b: 5, d: 4}\n    iex> Chord.Delta.calculate_delta(current_context, new_context)\n    %{\n      b: %{action: :modified, old_value: 2, value: 5},\n      c: %{action: :removed},\n      d: %{action: :added, value: 4}\n    }","ref":"Chord.Delta.html#calculate_delta/2-example"},{"type":"function","title":"Chord.Delta.format_delta/2","doc":"Formats a delta map for external communication or storage.\n\nUses the configured delta formatter to transform the delta map into a desired format.\nBy default, it uses `Chord.Delta.Formatter.Default`.","ref":"Chord.Delta.html#format_delta/2"},{"type":"function","title":"Parameters - Chord.Delta.format_delta/2","doc":"- `delta` (map): The delta to format.\n  - `context_id` (any): The context associated with the delta.","ref":"Chord.Delta.html#format_delta/2-parameters"},{"type":"function","title":"Returns - Chord.Delta.format_delta/2","doc":"- (list): The formatted delta, as determined by the formatter module.","ref":"Chord.Delta.html#format_delta/2-returns"},{"type":"function","title":"Example - Chord.Delta.format_delta/2","doc":"iex> delta = %{a: %{action: :added, value: 1}, b: %{action: :removed}}\n    iex> Chord.Delta.format_delta(delta, \"game:1\")\n    [\n      %{key: :a, action: :added, value: 1, context: \"game:1\"},\n      %{key: :b, action: :removed, context: \"game:1\"}\n    ]","ref":"Chord.Delta.html#format_delta/2-example"},{"type":"function","title":"Chord.Delta.merge_deltas/1","doc":"Merges a list of deltas into a single delta.\n\nCombines multiple delta maps to produce a unified view of changes. The merging logic ensures:\n- If any delta marks a key as `:removed`, it takes precedence.\n- If multiple deltas modify the same key, the `old_value` is taken from the first modification,\n  and the `value` from the last.\n- Otherwise, the latest change is retained.","ref":"Chord.Delta.html#merge_deltas/1"},{"type":"function","title":"Parameters - Chord.Delta.merge_deltas/1","doc":"- `delta_list` (list of maps): A list of deltas to merge.","ref":"Chord.Delta.html#merge_deltas/1-parameters"},{"type":"function","title":"Returns - Chord.Delta.merge_deltas/1","doc":"- (map): A single delta representing the merged changes.","ref":"Chord.Delta.html#merge_deltas/1-returns"},{"type":"function","title":"Example - Chord.Delta.merge_deltas/1","doc":"iex> delta1 = %{a: %{action: :added, value: 1}, b: %{action: :modified, old_value: 2, value: 3}}\n    iex> delta2 = %{b: %{action: :removed}, c: %{action: :added, value: 4}}\n    iex> Chord.Delta.merge_deltas([delta1, delta2])\n    %{\n      a: %{action: :added, value: 1},\n      b: %{action: :removed},\n      c: %{action: :added, value: 4}\n    }","ref":"Chord.Delta.html#merge_deltas/1-example"},{"type":"behaviour","title":"Chord.Delta.Formatter.Behaviour","doc":"Defines the behaviour for delta formatters.\n\nThis behaviour allows developers to implement custom delta formatting logic\ntailored to their application's requirements. By adhering to this behaviour,\ndevelopers can ensure that their formatter modules integrate seamlessly with Chord's\ndelta management system.","ref":"Chord.Delta.Formatter.Behaviour.html"},{"type":"behaviour","title":"Example Implementation - Chord.Delta.Formatter.Behaviour","doc":"defmodule MyCustomFormatter do\n      @behaviour Chord.Delta.Formatter.Behaviour\n\n      @impl true\n      def format(delta, context_id) do\n        Enum.map(delta, fn {key, change} ->\n          %{\n            key: key,\n            context: context_id,\n            action: change.action,\n            details: change\n          }\n        end)\n      end\n    end","ref":"Chord.Delta.Formatter.Behaviour.html#module-example-implementation"},{"type":"callback","title":"Chord.Delta.Formatter.Behaviour.format/2","doc":"","ref":"Chord.Delta.Formatter.Behaviour.html#c:format/2"},{"type":"module","title":"Chord.Delta.Formatter.Default","doc":"Default implementation for delta formatting.\n\nThis module provides a straightforward and opinionated way to format deltas into a list of maps.\nEach map describes a change to a specific key in the context, including metadata such as the action\nperformed (e.g., `:added`, `:modified`, `:removed`) and the associated `context_id`.\n\nThe formatting is intended to be extensible, allowing developers to substitute this formatter\nwith a custom implementation by defining their own module and setting it in the application configuration.","ref":"Chord.Delta.Formatter.Default.html"},{"type":"module","title":"Example - Chord.Delta.Formatter.Default","doc":"iex> delta = %{\n    ...>   a: %{action: :added, value: 1},\n    ...>   b: %{action: :modified, old_value: 2, value: 3},\n    ...>   c: %{action: :removed}\n    ...> }\n    iex> context_id = \"game:1\"\n    iex> Chord.Delta.Formatter.Default.format(delta, context_id)\n    [\n      %{key: :a, action: :added, value: 1, context: \"game:1\"},\n      %{key: :b, action: :modified, old_value: 2, value: 3, context: \"game:1\"},\n      %{key: :c, action: :removed, context: \"game:1\"}\n    ]","ref":"Chord.Delta.Formatter.Default.html#module-example"},{"type":"function","title":"Chord.Delta.Formatter.Default.format/2","doc":"Formats the given delta into a standardized list of maps.\n\nThis implementation adds the `context_id` to each change and formats changes as follows:\n- For `:added`, includes the new `value`.\n- For `:modified`, includes both the `old_value` and the new `value`.\n- For `:removed`, includes only the `key` and `context_id`.","ref":"Chord.Delta.Formatter.Default.html#format/2"},{"type":"function","title":"Parameters - Chord.Delta.Formatter.Default.format/2","doc":"- `delta` (map): The delta map representing changes to the context.\n  - `context_id` (any): The context ID to associate with each change.","ref":"Chord.Delta.Formatter.Default.html#format/2-parameters"},{"type":"function","title":"Returns - Chord.Delta.Formatter.Default.format/2","doc":"- (list of maps): A list of formatted changes.","ref":"Chord.Delta.Formatter.Default.html#format/2-returns"},{"type":"function","title":"Example - Chord.Delta.Formatter.Default.format/2","doc":"iex> delta = %{\n    ...>   a: %{action: :added, value: 1},\n    ...>   b: %{action: :modified, old_value: 2, value: 3},\n    ...>   c: %{action: :removed}\n    ...> }\n    iex> Chord.Delta.Formatter.Default.format(delta, \"game:1\")\n    [\n      %{key: :a, action: :added, value: 1, context: \"game:1\"},\n      %{key: :b, action: :modified, old_value: 2, value: 3, context: \"game:1\"},\n      %{key: :c, action: :removed, context: \"game:1\"}\n    ]","ref":"Chord.Delta.Formatter.Default.html#format/2-example"},{"type":"module","title":"Chord.Utils.Context.MapTransform","doc":"Utilities for manipulating nested maps, including deep merging.","ref":"Chord.Utils.Context.MapTransform.html"},{"type":"function","title":"Chord.Utils.Context.MapTransform.deep_update/2","doc":"Updates a deeply nested map using another map of updates.","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2"},{"type":"function","title":"Parameters - Chord.Utils.Context.MapTransform.deep_update/2","doc":"- `original` (map): The original map to be updated.\n- `updates` (map): A map containing the updates to be applied.","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2-parameters"},{"type":"function","title":"Notes - Chord.Utils.Context.MapTransform.deep_update/2","doc":"- Preserves the structure of the original map while applying the updates.\n- Creates missing keys in the original map if they are present in the updates.","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2-notes"},{"type":"function","title":"Examples - Chord.Utils.Context.MapTransform.deep_update/2","doc":"iex> alias Chord.Utils.Context.MapTransform\n    iex> original = %{users: %{user_a: %{name: \"Alice\", age: 30}, user_b: %{name: \"Bob\", age: 25}}}\n    iex> updates = %{users: %{user_b: %{age: 26}}}\n    iex> MapTransform.deep_update(original, updates)\n    %{users: %{user_a: %{name: \"Alice\", age: 30}, user_b: %{name: \"Bob\", age: 26}}}\n\n    iex> original = %{}\n    iex> updates = %{users: %{user_a: %{profile: %{name: \"Alice\"}}}}\n    iex> MapTransform.deep_update(original, updates)\n    %{users: %{user_a: %{profile: %{name: \"Alice\"}}}}","ref":"Chord.Utils.Context.MapTransform.html#deep_update/2-examples"},{"type":"behaviour","title":"Chord.Utils.Redis.Behaviour","doc":"Defines the behavior for Redis client operations.","ref":"Chord.Utils.Redis.Behaviour.html"},{"type":"callback","title":"Chord.Utils.Redis.Behaviour.command/1","doc":"","ref":"Chord.Utils.Redis.Behaviour.html#c:command/1"},{"type":"module","title":"Chord.Utils.Redis.Client","doc":"Wrapper around Redis commands to abstract interaction and enable easier testing.","ref":"Chord.Utils.Redis.Client.html"},{"type":"function","title":"Chord.Utils.Redis.Client.command/1","doc":"Executes a Redis command.\n\nDepending on the configuration, this function will:\n- Interact with the named Redix process (using `Process.whereis/1`).\n- Use a mock module for testing.","ref":"Chord.Utils.Redis.Client.html#command/1"},{"type":"function","title":"Parameters - Chord.Utils.Redis.Client.command/1","doc":"- `command` (list): The Redis command to execute.","ref":"Chord.Utils.Redis.Client.html#command/1-parameters"},{"type":"function","title":"Returns - Chord.Utils.Redis.Client.command/1","doc":"- `{:ok, result}` on success.\n  - `{:error, reason}` on failure.","ref":"Chord.Utils.Redis.Client.html#command/1-returns"},{"type":"module","title":"Chord.Utils.Time","doc":"Default implementation of the `Chord.Utils.Time.Behaviour`.\n\nProvides time utilities for the library, leveraging Erlang's built-in\ntime functions.","ref":"Chord.Utils.Time.html"},{"type":"module","title":"Configuration - Chord.Utils.Time","doc":"By default, this module is used for time-based operations. If you wish to\nuse a custom implementation, define a module adhering to the\n`Chord.Utils.Time.Behaviour` and configure it in your application:\n\n    config :chord, :time, MyCustomTimeProvider","ref":"Chord.Utils.Time.html#module-configuration"},{"type":"module","title":"Example - Chord.Utils.Time","doc":"iex> Chord.Utils.Time.current_time(:second)\n    1672531200","ref":"Chord.Utils.Time.html#module-example"},{"type":"function","title":"Chord.Utils.Time.current_time/1","doc":"Returns the current time in the specified unit.","ref":"Chord.Utils.Time.html#current_time/1"},{"type":"function","title":"Parameters - Chord.Utils.Time.current_time/1","doc":"- `unit` (:second | :millisecond): The desired unit for the time value.","ref":"Chord.Utils.Time.html#current_time/1-parameters"},{"type":"function","title":"Returns - Chord.Utils.Time.current_time/1","doc":"- The current time as an integer, in the specified unit.","ref":"Chord.Utils.Time.html#current_time/1-returns"},{"type":"function","title":"Example - Chord.Utils.Time.current_time/1","doc":"iex> Chord.Utils.Time.current_time(:second)\n    1672531200\n\n    iex> Chord.Utils.Time.current_time(:millisecond)\n    1672531200000","ref":"Chord.Utils.Time.html#current_time/1-example"},{"type":"behaviour","title":"Chord.Utils.Time.Behaviour","doc":"Defines the behavior for time utilities used by the library.\n\nThis behavior allows developers to implement custom time providers,\nmaking it easier to handle time-based operations (e.g., cleanup) and mock\ntime in tests.","ref":"Chord.Utils.Time.Behaviour.html"},{"type":"behaviour","title":"Example - Chord.Utils.Time.Behaviour","doc":"defmodule MyCustomTimeProvider do\n      @behaviour Chord.Utils.Time.Behaviour\n\n      @impl true\n      def current_time(:second) do\n        # Custom logic for returning current time\n        DateTime.utc_now() |> DateTime.to_unix(:second)\n      end\n\n      @impl true\n      def current_time(:millisecond) do\n        # Custom logic for millisecond precision\n        DateTime.utc_now() |> DateTime.to_unix(:millisecond)\n      end\n    end","ref":"Chord.Utils.Time.Behaviour.html#module-example"},{"type":"callback","title":"Chord.Utils.Time.Behaviour.current_time/1","doc":"","ref":"Chord.Utils.Time.Behaviour.html#c:current_time/1"},{"type":"extras","title":"README","doc":"Welcome to **Chord** - a flexible and powerful Elixir library designed to simplify context management and delta tracking in your distributed or real-time applications.\n\n[![Hex.pm](https://img.shields.io/hexpm/v/chord.svg)](https://hex.pm/packages/chord) [![Documentation](https://img.shields.io/badge/documentation-hexdocs-blue)](https://hexdocs.pm/chord)","ref":"readme.html"},{"type":"extras","title":"Why Chord? - README","doc":"When you need a solution for real-time state synchronization, partial updates, and efficient cleanup, Chord strikes the perfect note! Here’s what makes Chord special:\n\n- **Seamless state sync**: Keep your clients up-to-date with full context or delta-based updates.\n- **Customizable backend**: Use ETS, Redis, or your own backend implementation.\n- **Flexible delta formatting**: Define how your updates are structured.\n- **Periodic cleanup**: Automatically clear stale contexts or deltas.\n- **Developer-friendly APIs**: Simple, consistent, and easy-to-use APIs.\n- **Context export and restore**: Export contexts to or restore them from external providers.\n- **Partial updates**: Apply updates to specific fields within a context.\n- **Delta Tracking**: Efficiently track and retrieve state changes.\n- **Flexible architecture**: Chord works in both stateful (via GenServer) and stateless modes (direct calls to backends like Redis or ETS). This flexibility makes it easier to adapt Chord to a variety of use cases.\n\n---","ref":"readme.html#why-chord"},{"type":"extras","title":"🚀 Getting started - README","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Install the library - README","doc":"Add Chord to your Mix dependencies:\n\n```elixir\ndef deps do\n  [\n    {:chord, \"~> 0.1.4\"}\n  ]\nend\n```\n\nRun:\n\n```bash\nmix deps.get\n```","ref":"readme.html#install-the-library"},{"type":"extras","title":"Configure Chord - README","doc":"Add your desired configuration in `config/config.exs`:\n\n```elixir\nconfig :chord,\n  backend: Chord.Backend.ETS,                     # Choose your backend (Redis, ETS, etc.)\n  context_auto_delete: false,                     # Enable or disable auto-deletion of old contexts\n  context_ttl: 6 * 60 * 60,                       # Time-to-live for contexts\n  delta_ttl: 24 * 60 * 60,                        # Time-to-live for deltas\n  delta_threshold: 100,                           # Number of deltas to retain\n  delta_formatter: Chord.Delta.Formatter.Default, # Format for deltas\n  time_provider: Chord.Utils.Time,                # Time provider for consistent timestamps\n  export_callback: nil,                           # Optional: Define a callback for exporting contexts\n  context_external_provider: nil                  # Optional: Define a function for fetching external contexts\n```\n\n---","ref":"readme.html#configure-chord"},{"type":"extras","title":"How to use Chord - README","doc":"In Chord, a **context** is basically a container for state. The term **“context”** might mean different things in various fields, but in Chord, it specifically means a **container for state**. Here are some examples to explain this idea:\n\n- In a **chat application**, a context could be a group chat, including its details (e.g., participants, topic), and messages.\n- In a **game session**, a context might hold the game’s state, like player positions, scores, and progress.\n- In a **collaborative document editor**, a context could be the document’s state, keeping track of edits, updates, and collaborators.\n\nWith this understanding of the term, let's look at some practical examples.","ref":"readme.html#how-to-use-chord"},{"type":"extras","title":"Setting a context - README","doc":"Define the global context and track changes with deltas.\n\n```elixir\nChord.set_context(\"user:123\", %{status: \"online\", metadata: %{theme: \"light\", language: \"en-US\"}})\n{:ok,\n %{\n   context: %{\n     version: 1,\n     context: %{\n       status: \"online\",\n       metadata: %{language: \"en-US\", theme: \"light\"}\n     },\n     inserted_at: 1737570501,\n     context_id: \"user:123\"\n   },\n   delta: %{\n     version: 1,\n     delta: %{\n       status: %{value: \"online\", action: :added},\n       metadata: %{value: %{language: \"en-US\", theme: \"light\"}, action: :added}\n     },\n     inserted_at: 1737570501,\n     context_id: \"user:123\"\n   }\n }}\n```","ref":"readme.html#setting-a-context"},{"type":"extras","title":"Updating a context - README","doc":"Updates a portion of the global context associated with a specific identifier.\nThis function allows for partial modifications without affecting the entire context.\n\n```elixir\nChord.update_context(\"user:123\", %{metadata: %{theme: \"dark\"}})\n{:ok,\n %{\n   context: %{\n     version: 2,\n     context: %{status: \"online\", metadata: %{language: \"en-US\", theme: \"dark\"}},\n     inserted_at: 1737570583,\n     context_id: \"user:123\"\n   },\n   delta: %{\n     version: 2,\n     delta: %{\n       metadata: %{\n         theme: %{value: \"dark\", action: :modified, old_value: \"light\"}\n       }\n     },\n     inserted_at: 1737570583,\n     context_id: \"user:123\"\n   }\n }}\n```","ref":"readme.html#updating-a-context"},{"type":"extras","title":"Getting a context - README","doc":"Fetches the current state for a specified identifier.\n\n```elixir\nChord.get_context(\"user:123\")\n{:ok,\n %{\n   version: 2,\n   context: %{status: \"online\", metadata: %{language: \"en-US\", theme: \"dark\"}},\n   inserted_at: 1737570583,\n   context_id: \"user:123\"\n }}\n```","ref":"readme.html#getting-a-context"},{"type":"extras","title":"Synchronizing state - README","doc":"Synchronize the state for a given identifier.\nDepending on the version the client has, it will receive either the full context, only the changes (deltas), or a notification that there are no updates.\n\n```elixir\nChord.sync_context(\"user:123\", nil)\n{:full_context,\n %{\n   version: 2,\n   context: %{status: \"online\", metadata: %{language: \"en-US\", theme: \"dark\"}},\n   inserted_at: 1737570583,\n   context_id: \"user:123\"\n }}\n\nChord.sync_context(\"user:123\", 1)\n{:delta,\n %{\n   version: 2,\n   delta: %{\n     metadata: %{theme: %{value: \"dark\", action: :modified, old_value: \"light\"}}\n   },\n   inserted_at: 1737570583,\n   context_id: \"user:123\"\n }}\n\nChord.sync_context(\"user:123\", 2)\n{:no_change, 2}\n```","ref":"readme.html#synchronizing-state"},{"type":"extras","title":"Exporting a context - README","doc":"Save the current context for a specific identifier to external storage using the configured export callback.\n\n#### Defining the export callback\nTo enable the export functionality, you need to define a callback function in your application. This function will handle how the context is exported (e.g., saving it to a database). Here’s an example:\n\n```elixir\ndefmodule MyApp.ContextExporter do\n  @moduledoc \"\"\"\n  Handles exporting contexts to external storage.\n  \"\"\"\n\n  @spec export_context(map()) :: :ok | {:error, term()}\n  def export_context(context_data) do\n    %{context_id: context_id, version: verion, context: context} = context_data\n    # Example: Save context_data to an external database or storage\n    case ExternalStorage.save(context_id, context, version) do\n      :ok -> :ok\n      {:error, reason} -> {:error, reason}\n    end\n  end\nend\n```\n\n#### Configure the export callback\nNext, configure the export callback in your application’s environment. This tells Chord how to handle context exports.\n\n```elixir\n# config/config.exs\nconfig :chord, :export_callback, &MyApp.ContextExporter.export_context/1\n```\n\n#### Use Chord.export_context/1\nOnce the callback is configured, you can use function to export a specific context to external storage:\n\n```elixir\nChord.export_context(\"user:123\")\n:ok\n```","ref":"readme.html#exporting-a-context"},{"type":"extras","title":"Deleting a context - README","doc":"Removes the entire context and its associated deltas.\n\n```elixir\nChord.delete_context(\"user:123\")\n:ok\n```","ref":"readme.html#deleting-a-context"},{"type":"extras","title":"Restoring a context - README","doc":"Retrieve and restore a context from an external provider to the current backend.\n\n#### Define the restore callback\nFirst, define a module and function that will handle the logic for retrieving a context. For example:\n\n```elixir\ndefmodule MyApp.ContextRestorer do\n  @moduledoc \"\"\"\n  Handles restoring contexts from external storage.\n  \"\"\"\n\n  @spec restore_context(String.t()) :: {:ok, map()} | {:error, term()}\n  def restore_context(context_id) do\n    # Example: Retrieve the context from a database or other storage system\n    case ExternalStorage.get(context_id) do\n      {:ok, %{context: context, version: version}} -> {:ok, %{context: context, version: version}}\n      {:error, reason} -> {:error, reason}\n    end\n  end\nend\n```\n\n#### Configure the restore callback\nNext, configure the restore callback in your application’s environment. This tells Chord how to handle context restoration:\n\n```elixir\n# config/config.exs\nconfig :chord, :context_external_provider, &MyApp.ContextRestorer.restore_context/1\n```\n\n#### Use Chord.restore_context/1\nOnce the callback is configured, you can use function to retrieve and restore a specific context:\n\n```elixir\nChord.restore_context(\"user:1234\")\n{:ok,\n %{\n   version: 10,\n   context: %{source: \"external storage provider\"},\n   inserted_at: 1737464001,\n   context_id: \"user:1234\"\n }}\n```","ref":"readme.html#restoring-a-context"},{"type":"extras","title":"Cleanup operations - README","doc":"Chord provides cleanup functionality to remove stale contexts and deltas. To enable and configure this feature, add the following settings to your application configuration:\n\n#### Configuration options\n\n```elixir\nconfig :chord,\n  context_auto_delete: true, # Enable or disable auto-deletion of old contexts\n  context_ttl: 6 * 60 * 60,  # Time-to-live for contexts (in seconds)\n  delta_ttl: 24 * 60 * 60,   # Time-to-live for deltas (optional, in seconds)\n  delta_threshold: 100       # Number of delta versions to retain (optional)\n```\n\n#### How it works\n- Context cleanup:\n  - Set **context_auto_delete: true** to enable context cleanup.\n  - Configure **context_ttl** to define how long contexts should remain in memory before being deleted. The value must be in seconds, as timestamps are created with second-level precision.\n  - When a context is deleted, all associated deltas are automatically cleaned up as well.\n\n- Delta cleanup:\n  - To clean deltas by age, set **delta_ttl** to specify the maximum time deltas should remain in memory.\n  - To clean deltas by number, set **delta_threshold** to define the maximum number of deltas to retain.\n\n\n#### Example usage\nRun the cleanup process manually with:\n\n```elixir\nChord.cleanup(limit: 50)\n```\n\n#### Future plans\nIn the future, you’ll be able to configure the time unit (e.g., seconds, milliseconds), which will be used when generating timestamps.\nThis configuration will apply to the function responsible for providing the current time, primarily used when setting inserted_at during data storage in memory.\n\nIf the configured time unit is set to seconds, related configurations such as context_ttl and delta_ttl will also need to be specified in seconds to ensure consistency.","ref":"readme.html#cleanup-operations"},{"type":"extras","title":"Managing the cleanup server - README","doc":"Start and manage the Cleanup Server for automated periodic cleanup:\n\n```elixir\n{:ok, _pid} = Chord.start_cleanup_server(interval: :timer.minutes(30))\nChord.update_cleanup_interval(:timer.minutes(60))\nChord.update_cleanup_backend_opts(limit: 100)\nChord.stop_cleanup_server()\n```\n\n---","ref":"readme.html#managing-the-cleanup-server"},{"type":"extras","title":"Customization - README","doc":"","ref":"readme.html#customization"},{"type":"extras","title":"Backends - README","doc":"Chord supports multiple backends out-of-the-box:\n\n- **ETS** (In-Memory): No additional setup is required.\n- **Redis** (Distributed): Requires a Redis instance and some configuration.\n\n#### Using Redis as a backend\nTo use Redis as the backend for Chord, follow these steps:\n\n1. **Start Redis**: Ensure a Redis server is running.\n2. **Set up the Redis connection**: Start a Redis connection process using the Redix library, which is included with Chord:\n\n```elixir\n{:ok, _} = Redix.start_link(\"redis://localhost:6379\", name: :my_redis)\n```\n\n3. **Configure Chord to use Redis**: Set the Redis client and backend in your application’s\n\n```elixir\n# config/config.exs\nconfig :chord,\n  backend: Chord.Backend.Redis,\n  redis_client: :my_redis\n```\n\nYou can also implement your own backend by adhering to the `Chord.Backend.Behaviour`.","ref":"readme.html#backends"},{"type":"extras","title":"Delta formatters - README","doc":"Chord provides the ability to define custom delta formatters by implementing the `Chord.Delta.Formatter.Behaviour`. This feature is useful for tailoring how deltas (changes) are formatted to suit your application’s requirements.\n\n#### Defining a custom delta formatter\nTo define a custom delta formatter, create a module that implements the `Chord.Delta.Formatter.Behaviour`:\n\n```elixir\ndefmodule MyApp.CustomFormatter do\n  @moduledoc \"\"\"\n  A custom delta formatter for Chord, demonstrating how to implement the behavior.\n  \"\"\"\n\n  @behaviour Chord.Delta.Formatter.Behaviour\n\n  @impl true\n  def format(delta, context_id) do\n    timestamp = DateTime.utc_now() |> DateTime.to_iso8601()\n\n    Enum.map(delta, fn {key, change} ->\n      base = %{\n        timestamp: timestamp,\n        context_id: context_id,\n        field: key,\n        action: change.action\n      }\n\n      case change.action do\n        :added ->\n          Map.put(base, :new_value, change.value)\n\n        :modified ->\n          Map.merge(base, %{old_value: change.old_value, new_value: change.value})\n\n        :removed ->\n          base\n      end\n    end)\n  end\nend\n```\n\n#### Configuring Chord to use your delta formatter\nOnce you’ve defined your custom formatter, configure Chord to use it by setting it in the application environment:\n\n```elixir\n# config/config.exs\nconfig :chord, :delta_formatter, MyApp.CustomFormatter\n```\n\n#### Example usage\n\n```elixir\ndelta = %{\n  a: %{action: :added, value: 1},\n  b: %{action: :modified, old_value: 2, value: 3},\n  c: %{action: :removed}\n}\n\ncontext_id = \"game:42\"\n\nformatted_delta = MyApp.CustomFormatter.format(delta, context_id)\n\nIO.inspect(formatted_delta)\n[\n  %{\n    timestamp: \"2025-01-22T15:00:00Z\",\n    context_id: \"game:42\",\n    field: :a,\n    action: :added,\n    new_value: 1\n  },\n  %{\n    timestamp: \"2025-01-22T15:00:00Z\",\n    context_id: \"game:42\",\n    field: :b,\n    action: :modified,\n    old_value: 2,\n    new_value: 3\n  },\n  %{\n    timestamp: \"2025-01-22T15:00:00Z\",\n    context_id: \"game:42\",\n    field: :c,\n    action: :removed\n  }\n]\n```","ref":"readme.html#delta-formatters"},{"type":"extras","title":"Custom time provider - README","doc":"Chord allows you to define custom time provider by implementing the `Chord.Utils.Time.Behaviour`. This feature is useful for customizing time-based operations, such as timestamp generation and for mocking time in tests.\n\n#### Defining a custom time provider\nTo define your custom time provider, create a module that implements the `Chord.Utils.Time.Behaviour`:\n\n```elixir\ndefmodule MyApp.CustomTimeProvider do\n  @moduledoc \"\"\"\n  A custom time provider for Chord, demonstrating how to implement the behavior.\n  \"\"\"\n\n  @behaviour Chord.Utils.Time.Behaviour\n\n  @impl true\n  def current_time(:second) do\n    # Example: Use a custom logic for time in seconds\n    DateTime.utc_now() |> DateTime.to_unix(:second)\n  end\n\n  @impl true\n  def current_time(:millisecond) do\n    # Example: Use a custom logic for time in milliseconds\n    DateTime.utc_now() |> DateTime.to_unix(:millisecond)\n  end\nend\n```\n\n#### Configuring Chord to use your time provider\n\n```elixir\n# config/config.exs\nconfig :chord, :time_provider, MyApp.CustomTimeProvider\n```\n\n---","ref":"readme.html#custom-time-provider"},{"type":"extras","title":"Benchmark results: Redis and ETS performance - README","doc":"Chord has been tested to ensure solid performance in both Redis (single-node setup for now, with plans for distributed scenarios) and ETS (in-memory, single-node applications). Here’s how it performs under various scenarios:","ref":"readme.html#benchmark-results-redis-and-ets-performance"},{"type":"extras","title":"Scenarios tested - README","doc":"","ref":"readme.html#scenarios-tested"},{"type":"extras","title":"1. Stateless operations - README","doc":"These scenarios simulate operations without maintaining a dedicated process per context. All updates, syncs and state modifications happen directly through the library’s API.\n\n- **Single context (50 participants):** Represents a single group chat or meeting with 50 participants frequently updating their status, typing indicators, or syncing state.\n- **Multiple contexts (100 contexts):** Simulates 100 independent group chats or meetings being updated simultaneously.","ref":"readme.html#1-stateless-operations"},{"type":"extras","title":"2. Stateful operations - README","doc":"These scenarios introduce a process per context (e.g., a GenServer for each group chat). Each participant interacts with this stateful process and the process uses Chord’s API to manage context.\n\n- **Single context (50 participants):** A single group chat or meeting managed by a GenServer, handling frequent updates and syncs from 50 participants.\n- **Multiple contexts (100 contexts):** Simulates 100 group chats or meetings, each managed by its own GenServer, handling participant interactions.","ref":"readme.html#2-stateful-operations"},{"type":"extras","title":"Results - README","doc":"","ref":"readme.html#results"},{"type":"extras","title":"Redis backend (single node) - README","doc":"> **Note:** Redis benchmarks were conducted in a single-node configuration to evaluate baseline performance. While Redis is designed for distributed systems, a fully distributed environment is not yet implemented in the benchmark script. Plans are underway to expand the benchmarking script to support distributed scenarios.\n\n| **Scenario**                       | **Operations/sec** | **Average Time** | **Notes**                                        |\n|------------------------------------|--------------------|------------------|--------------------------------------------------|\n| Stateless - Single Context (50)    | 92.89 ops/s        | 10.77 ms         | Handles concurrent operations efficiently.       |\n| Stateful - Single Context (50)     | 18.80 ops/s        | 53.19 ms         | Performance impacted by GenServer overhead.      |\n| Stateful - Multiple Contexts (100) | 1.72 ops/s         | 581.46 ms        | Slower due to process sync overhead.             |\n| Stateless - Multiple Contexts (100)| 1.57 ops/s         | 635.29 ms        | Poor throughput under high multi-context load.   |","ref":"readme.html#redis-backend-single-node"},{"type":"extras","title":"ETS Backend (In-Memory, Single-Node) - README","doc":"| **Scenario**                       | **Operations/sec** | **Average Time** | **Notes**                                        |\n|------------------------------------|--------------------|------------------|--------------------------------------------------|\n| Stateless - Single Context (50)    | 230.61 ops/s       | 4.34 ms          | Extremely fast for single-node setups.           |\n| Stateful - Single Context (50)     | 54.34 ops/s        | 18.40 ms         | GenServer overhead slows performance.            |\n| Stateful - Multiple Contexts (100) | 5.66 ops/s         | 176.81 ms        | Scales well but slower with 100 contexts.        |\n| Stateless - Multiple Contexts (100)| 4.69 ops/s         | 213.18 ms        | Limited scalability for multi-context updates.   |","ref":"readme.html#ets-backend-in-memory-single-node"},{"type":"extras","title":"Key insights - README","doc":"","ref":"readme.html#key-insights"},{"type":"extras","title":"Redis: - README","doc":"- **Single-node performance**: Reflects the baseline for Redis's capability, with potential for distributed scaling in the future.\n- **Stateless operations** Outperform stateful ones when multiple clients update a single context concurrently. The absence of GenServer synchronization overhead makes Redis particularly well-suited for high-frequency, multi-client updates to shared contexts.\n- **Stateful performance bottlenecks**: Syncing multiple contexts (100) causes significant performance degradation due to process synchronization and network overhead.\n- **Future improvements**: A distributed Redis setup would allow benchmarking its true scalability and potential for handling high-throughput, multi-context scenarios.","ref":"readme.html#redis"},{"type":"extras","title":"ETS: - README","doc":"- **Optimal for single-node applications**: Outshines Redis in single-node scenarios, particularly in stateless operations where 50 participants concurrently update a single context, achieving 230.61 ops/sec with just 4.34 ms latency.\n- **GenServer overhead**: Stateful operations see reduced performance due to process-based synchronization, especially with many contexts (e.g., 100).\n- **Scalability limitations:**: While ETS is efficient for localized, single-node setups, it struggles with multi-context workloads, where its performance drops to 4.69 ops/sec for stateless and 5.66 ops/sec for stateful scenarios.","ref":"readme.html#ets"},{"type":"extras","title":"Choosing between stateless and stateful - README","doc":"","ref":"readme.html#choosing-between-stateless-and-stateful"},{"type":"extras","title":"Stateless: - README","doc":"- Directly interacts with Chord’s API, bypassing the need for per-context processes.\n- **Best for**: High-concurrency scenarios where multiple clients update a single shared context. Performance degrades under high multi-context workloads, particularly in Redis.","ref":"readme.html#stateless"},{"type":"extras","title":"Stateful: - README","doc":"- Manages a dedicated GenServer per context (e.g., per group chat or meeting).\n- **Best for**: Scenarios requiring additional application-level state or business logic. However, high-concurrency and multi-context workloads may lead to significant performance degradation.","ref":"readme.html#stateful"},{"type":"extras","title":"Device Information - README","doc":"| Property                   | Value                    |\n|----------------------------|--------------------------|\n| **Operating System**       | macOS                    | \n| **CPU Information**        | Apple M4 Pro             |\n| **Number of Cores**        | 12                       |\n| **Available Memory**       | 24 GB                    |\n| **Elixir Version**         | 1.17.3                   |\n| **Erlang Version**         | 27.1.2                   |\n| **JIT Enabled**            | True                     |\n\n**Benchmark Suite Configuration:**\n- **Warmup:** 2 seconds\n- **Execution Time:** 5 seconds\n- **Parallel:** 1\n- **Inputs:** Data\n\n---","ref":"readme.html#device-information"},{"type":"extras","title":"Contributing - README","doc":"Contributions from the community are welcome to make Chord even better! Whether it's fixing bugs, improving documentation, or adding new features, your help is greatly appreciated.","ref":"readme.html#contributing"},{"type":"extras","title":"How to contribute - README","doc":"1. Fork the repository.\n2. Create a new branch for your changes.\n3. Make your changes and test them thoroughly.\n4. Submit a pull request with a clear description of your changes.\n\nFeel free to open issues for discussion or if you need help. Together, we can build something amazing!\n\n---","ref":"readme.html#how-to-contribute"},{"type":"extras","title":"Testing - README","doc":"Chord comes with a robust suite of tests to ensure reliability. Run tests with:\n\n```bash\nmix test\n```\n\n---\n\n🎵 *\"Let Chord orchestrate your state management with precision and elegance.\"*","ref":"readme.html#testing"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/).\n\n## [0.1.4] - 2025-01-22","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Enhanced `README.md` with detailed examples for using the Chord library.\n- Unit tests for `calculate_delta` to handle `nil` values and ensure proper behavior when handling nested maps.\n- Project logo","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated `delta_ttl` configuration to use seconds for consistency across the library.\n- Improved `calculate_delta` function to:\n  - Handle `nil` values appropriately.\n  - Mark changes as `:modified` when the old value is `nil`.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Bug in Redis backend:\n  - `fetch_delta_counts/1` now correctly splits keys using `String.split/3` with `parts: 3` to handle keys with more than two colons.\n\n## [0.1.3] - 2025-01-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Updated README.md to include missing details.\n\n## [0.1.2] - 2025-01-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed delta merging logic to correctly handle nested maps and ensure all changes are applied.\n\n## [0.1.1] - 2025-01-05","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Corrected Redis configuration example in documentation.\n- Fixed an issue in `Chord.Delta.calculate_delta/2` to correctly handle nested maps and avoid returning empty maps for unchanged nested structures.\n\n## [0.1.0] - 2024-12-31","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of the Chord library.\n- Real-time state synchronization with full and delta-based updates.\n- Support for ETS and Redis backends.\n- Developer-friendly APIs for setting, updating, and syncing contexts.\n- Periodic cleanup of stale contexts and deltas.\n- Support for context export and restore.\n- Benchmark results showcasing performance for ETS and Redis under stateless and stateful architectures.","ref":"changelog.html#added"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}